@@ -67,40 +67,16 @@ class SparseUnorderedWithDupsReaderStatusException : public StatusException {
 
 template <class BitmapType>
 SparseUnorderedWithDupsReader<BitmapType>::SparseUnorderedWithDupsReader(
-    stats::Stats* stats,
-    shared_ptr<Logger> logger,
-    StorageManager* storage_manager,
-    Array* array,
-    Config& config,
-    std::unordered_map<std::string, QueryBuffer>& buffers,
-    std::unordered_map<std::string, QueryBuffer>& aggregate_buffers,
-    Subarray& subarray,
-    Layout layout,
-    std::optional<QueryCondition>& condition,
-    DefaultChannelAggregates& default_channel_aggregates,
-    bool skip_checks_serialization)
+    stats::Stats* stats, shared_ptr<Logger> logger, StrategyParams& params)
     : SparseIndexReaderBase(
-          "sparse_unordered_with_dups",
-          stats,
-          logger->clone("SparseUnorderedWithDupsReader", ++logger_id_),
-          storage_manager,
-          array,
-          config,
-          buffers,
-          aggregate_buffers,
-          subarray,
-          layout,
-          condition,
-          default_channel_aggregates,
-          skip_checks_serialization,
-          false)
+          "sparse_unordered_with_dups", stats, logger, params, false)
     , tile_offsets_min_frag_idx_(std::numeric_limits<unsigned>::max())
     , tile_offsets_max_frag_idx_(0) {
   // Initialize memory budget variables.
   refresh_config();
 
-  // Get the setting that allows to partially load tile offsets. This is done
-  // for this reader only for now.
+  // Get the setting that allows to partially load tile offsets. This is
+  // done for this reader only for now.
   bool found = false;
   if (!config_
            .get<bool>(
@@ -119,7 +95,7 @@ SparseUnorderedWithDupsReader<BitmapType>::SparseUnorderedWithDupsReader(
 
 template <class BitmapType>
 bool SparseUnorderedWithDupsReader<BitmapType>::incomplete() const {
-  return !read_state_.done_adding_result_tiles_ ||
+  return !read_state_.done_adding_result_tiles() ||
          !result_tiles_leftover_.empty();
 }
 
@@ -146,9 +122,6 @@ void SparseUnorderedWithDupsReader<BitmapType>::refresh_config() {
 
 template <class BitmapType>
 Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
-  // Subarray is not known to be explicitly set until buffers are deserialized
-  include_coords_ = subarray_.is_set();
-
   auto timer_se = stats_->start_timer("dowork");
   stats_->add_counter("loop_num", 1);
 
@@ -173,10 +146,14 @@ Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
 
   // Handle empty array.
   if (fragment_metadata_.empty()) {
-    read_state_.done_adding_result_tiles_ = true;
+    read_state_.set_done_adding_result_tiles(true);
     return Status::Ok();
   }
 
+  // Subarray is not known to be explicitly set until buffers are deserialized
+  subarray_.reset_default_ranges();
+  include_coords_ = subarray_.is_set();
+
   // Load initial data, if not loaded already. Coords are only included if the
   // subarray is set.
   throw_if_not_ok(load_initial_data());
@@ -200,7 +177,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
 
       // No more tiles to process, done.
       if (result_tiles.empty()) {
-        assert(read_state_.done_adding_result_tiles_);
+        assert(read_state_.done_adding_result_tiles());
         break;
       }
 
@@ -275,9 +252,9 @@ void SparseUnorderedWithDupsReader<BitmapType>::load_tile_offsets_data() {
   bool initial_load =
       tile_offsets_min_frag_idx_ == std::numeric_limits<unsigned>::max() &&
       tile_offsets_max_frag_idx_ == 0;
-  uint64_t available_memory = array_memory_tracker_->get_memory_available() -
-                              array_memory_tracker_->get_memory_usage(
-                                  MemoryTracker::MemoryType::TILE_OFFSETS);
+  uint64_t available_memory =
+      array_memory_tracker_->get_memory_available() -
+      array_memory_tracker_->get_memory_usage(MemoryType::TILE_OFFSETS);
   auto& relevant_fragments = subarray_.relevant_fragments();
 
   if (!partial_tile_offsets_loading_) {
@@ -426,7 +403,7 @@ bool SparseUnorderedWithDupsReader<BitmapType>::add_result_tile(
   memory_used_for_coords_total_ += tiles_size;
 
   // Add the result tile.
-  result_tiles.emplace_back(f, t, frag_md);
+  result_tiles.emplace_back(f, t, frag_md, query_memory_tracker_);
 
   // Are all tiles loaded for this fragment.
   if (t == last_t) {
@@ -492,7 +469,7 @@ SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
         auto tile_num = fragment_metadata_[f]->tile_num();
 
         // Figure out the start index.
-        auto start = read_state_.frag_idx_[f].tile_idx_;
+        auto start = read_state_.frag_idx()[f].tile_idx_;
 
         // Add all tiles for this fragment.
         if (start == tile_num) {
@@ -526,7 +503,7 @@ SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
     logger_->debug("All result tiles loaded");
   }
 
-  read_state_.done_adding_result_tiles_ = done_adding_result_tiles;
+  read_state_.set_done_adding_result_tiles(done_adding_result_tiles);
 
   return result_tiles;
 }
@@ -800,7 +777,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::copy_offsets_tiles(
         // We might have a partially processed result tile from last run.
         auto min_pos_tile = 0;
         if (i == 0) {
-          min_pos_tile = read_state_.frag_idx_[rt->frag_idx()].cell_idx_;
+          min_pos_tile = read_state_.frag_idx()[rt->frag_idx()].cell_idx_;
         }
 
         auto max_pos_tile =
@@ -1278,7 +1255,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::copy_fixed_data_tiles(
         // We might have a partially processed result tile from last run.
         auto min_pos_tile = 0;
         if (i == 0) {
-          min_pos_tile = read_state_.frag_idx_[rt->frag_idx()].cell_idx_;
+          min_pos_tile = read_state_.frag_idx()[rt->frag_idx()].cell_idx_;
         }
 
         auto max_pos_tile =
@@ -1448,7 +1425,7 @@ SparseUnorderedWithDupsReader<BitmapType>::resize_fixed_results_to_copy(
 
   uint64_t initial_cell_offset = cells_copied(names);
   uint64_t first_tile_min_pos =
-      read_state_.frag_idx_[result_tiles[0]->frag_idx()].cell_idx_;
+      read_state_.frag_idx()[result_tiles[0]->frag_idx()].cell_idx_;
 
   return resize_fixed_result_tiles_to_copy(
       max_num_cells, initial_cell_offset, first_tile_min_pos, result_tiles);
@@ -1737,7 +1714,7 @@ bool SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
   if (result_tiles.size() > 0) {
     auto last_tile =
         (UnorderedWithDupsResultTile<BitmapType>*)result_tiles.back();
-    auto& frag_tile_idx = read_state_.frag_idx_[last_tile->frag_idx()];
+    auto& frag_tile_idx = read_state_.frag_idx()[last_tile->frag_idx()];
     last_tile_cells_copied = cell_offsets[result_tiles.size()] -
                              cell_offsets[result_tiles.size() - 1];
     if (frag_tile_idx.tile_idx_ == last_tile->tile_idx()) {
@@ -1748,18 +1725,20 @@ bool SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
 
   // Adjust tile index.
   for (auto rt : result_tiles) {
-    read_state_.frag_idx_[rt->frag_idx()] = FragIdx(rt->tile_idx() + 1, 0);
+    read_state_.set_frag_idx(rt->frag_idx(), FragIdx(rt->tile_idx() + 1, 0));
   }
 
   // If the last tile is not fully copied, save the cell index.
   if (result_tiles.size() > 0) {
     auto last_tile =
         (UnorderedWithDupsResultTile<BitmapType>*)result_tiles.back();
-    auto& frag_tile_idx = read_state_.frag_idx_[last_tile->frag_idx()];
     if (last_tile->result_num() != last_tile_cells_copied) {
-      frag_tile_idx.tile_idx_ = last_tile->tile_idx();
-      frag_tile_idx.cell_idx_ =
-          last_tile->pos_with_given_result_sum(0, last_tile_cells_copied) + 1;
+      read_state_.set_frag_idx(
+          last_tile->frag_idx(),
+          FragIdx(
+              last_tile->tile_idx(),
+              last_tile->pos_with_given_result_sum(0, last_tile_cells_copied) +
+                  1));
     }
   }
 
@@ -1827,7 +1806,7 @@ bool SparseUnorderedWithDupsReader<BitmapType>::copy_tiles(
   uint64_t var_buffer_size = 0;
   if (var_sized) {
     uint64_t first_tile_min_pos =
-        read_state_.frag_idx_[result_tiles[0]->frag_idx()].cell_idx_;
+        read_state_.frag_idx()[result_tiles[0]->frag_idx()].cell_idx_;
 
     // Adjust the offsets buffer and make sure all data fits.
     auto&& [caused_overflow, new_var_buffer_size, new_result_tiles_size] =
@@ -1958,7 +1937,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::process_aggregates(
 
         // The first tile might have already been processed by the last
         // computation. We only process a tile the first time.
-        if (i == 0 && read_state_.frag_idx_[rt->frag_idx()].cell_idx_ != 0) {
+        if (i == 0 && read_state_.frag_idx()[rt->frag_idx()].cell_idx_ != 0) {
           return Status::Ok();
         }
 
@@ -2025,9 +2004,10 @@ template <class BitmapType>
 void SparseUnorderedWithDupsReader<BitmapType>::end_iteration(
     ResultTilesList& result_tiles) {
   // Clear result tiles that are not necessary anymore.
-  while (!result_tiles.empty() &&
-         result_tiles.front().tile_idx() <
-             read_state_.frag_idx_[result_tiles.front().frag_idx()].tile_idx_) {
+  while (
+      !result_tiles.empty() &&
+      result_tiles.front().tile_idx() <
+          read_state_.frag_idx()[result_tiles.front().frag_idx()].tile_idx_) {
     const auto f = result_tiles.front().frag_idx();
 
     remove_result_tile(f, result_tiles, result_tiles.begin());
@@ -2050,31 +2030,9 @@ void SparseUnorderedWithDupsReader<BitmapType>::end_iteration(
 
 // Explicit template instantiations
 template SparseUnorderedWithDupsReader<uint8_t>::SparseUnorderedWithDupsReader(
-    stats::Stats*,
-    shared_ptr<Logger>,
-    StorageManager*,
-    Array*,
-    Config&,
-    std::unordered_map<std::string, QueryBuffer>&,
-    std::unordered_map<std::string, QueryBuffer>&,
-    Subarray&,
-    Layout,
-    std::optional<QueryCondition>&,
-    DefaultChannelAggregates&,
-    bool);
+    stats::Stats*, shared_ptr<Logger>, StrategyParams&);
 template SparseUnorderedWithDupsReader<uint64_t>::SparseUnorderedWithDupsReader(
-    stats::Stats*,
-    shared_ptr<Logger>,
-    StorageManager*,
-    Array*,
-    Config&,
-    std::unordered_map<std::string, QueryBuffer>&,
-    std::unordered_map<std::string, QueryBuffer>&,
-    Subarray&,
-    Layout,
-    std::optional<QueryCondition>&,
-    DefaultChannelAggregates&,
-    bool);
+    stats::Stats*, shared_ptr<Logger>, StrategyParams&);
 
 }  // namespace sm
 }  // namespace tiledb
